<?php
/**
 * This Plugin is subject to copyright protection. It is only for the use of
 * Shopgate GmbH customers, for the purpose of facilitating communication between
 * the IT systems of the customer and the IT systems of Shopgate GmbH via
 * www.Shopgate.com. Any reproduction, dissemination, public propagation,
 * processing or transfer to third parties is only permitted where we previously
 * consented thereto in writing. The provisions of paragraph 69 d,
 * sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of
 * the German Copyright Act shall remain unaffected.
 *
 *
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von
 * Kunden der Shopgate GmbH zum Zwecke der eigenen Kommunikation zwischen
 * dem IT-System des Kunden mit dem IT-System der Shopgate GmbH verwendet werden.
 * Eine darüber hinausgehende Vervielfältigung, Verbreitung, öffentliche
 * Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer
 * vorherigen schriftlichen Zustimmung zulässig. Die Regelungen
 * der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 */

/**
 * Helper class used by check_cart
 */
class ShopgateCartHelper {
	
	/** @var ShopgateConfigOxid */
	private $config;
	
	/** @var oxUser */
	private $oxUser;
	
	/**
	 * @param ShopgateConfigOxid $config
	 */
	public function __construct(ShopgateConfigOxid $config) {
		$this->config = $config;
	}
	
	/**
	 * @param oxUser $oxUser
	 *
	 * @return ShopgateCartCustomer
	 */
	public function buildCartCustomer(oxUser $oxUser) {
		$this->oxUser = $oxUser;
		
		$customer = new ShopgateCartCustomer();
		$customer->setCustomerGroups($this->getCustomerGroups());
		
		return $customer->utf8Encode($this->config->getEncoding());
	}
	
	/**
	 * @return ShopgateCartCustomerGroup[]
	 */
	private function getCustomerGroups() {
		$customerGroups = array();
		foreach (marm_shopgate::getUserGroupsByUser($this->oxUser) as $oxGroup) {
			$group = new ShopgateCartCustomerGroup();
			$group->setId($oxGroup->oxgroups__oxid->value);
			$customerGroups[$oxGroup->oxgroups__oxid->value] = $group;
		}
		
		return $customerGroups;
	}
	
	/**
	 * In case method getItems on the cart object returns not an array this method will return true
	 *
	 * @param ShopgateCart $cart
	 *
	 * @return bool
	 */
	public function isShoppingCartEmpty(ShopgateCart $cart) {
		$cartItems = $cart->getItems();
		
		if (!is_array($cartItems) || count($cartItems) == 0) {
			return true;
		}
		
		return false;
	}
}