<?php
/**
 * This Plugin is subject to copyright protection. It is only for the use of
 * Shopgate GmbH customers, for the purpose of facilitating communication between
 * the IT systems of the customer and the IT systems of Shopgate GmbH via
 * www.Shopgate.com. Any reproduction, dissemination, public propagation,
 * processing or transfer to third parties is only permitted where we previously
 * consented thereto in writing. The provisions of paragraph 69 d,
 * sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of
 * the German Copyright Act shall remain unaffected.
 *
 *
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von
 * Kunden der Shopgate GmbH zum Zwecke der eigenen Kommunikation zwischen
 * dem IT-System des Kunden mit dem IT-System der Shopgate GmbH verwendet werden.
 * Eine darüber hinausgehende Vervielfältigung, Verbreitung, öffentliche
 * Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer
 * vorherigen schriftlichen Zustimmung zulässig. Die Regelungen
 * der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 */

/**
 * Helper class used by get_customer
 */
class ShopgateCustomerExportHelper {
	
	/** @var ShopgateConfigOxid */
	private $config;
	
	/** @var oxUser */
	private $oxUser;
	
	/**
	 * @param ShopgateConfigOxid $config
	 */
	public function __construct(ShopgateConfigOxid $config) {
		$this->config = $config;
	}
	
	/**
	 * @param oxUser $oxUser
	 * @return ShopgateContainer|ShopgateCustomer
	 */
	public function buildExternalCustomer(oxUser $oxUser) {
		$this->oxUser = $oxUser;
		
		$customer = new ShopgateCustomer();
		$customer->setCustomerId(             $this->getCustomerId());
		$customer->setCustomerToken(          $this->getCustomerToken());
		$customer->setCustomerNumber(         $this->getCustomerNumber());
		$customer->setFirstName(              $this->getFirstName());
		$customer->setLastName(               $this->getLastName());
		$customer->setMail(                   $this->getMail());
		$customer->setPhone(                  $this->getPhone());
		$customer->setMobile(                 $this->getMobile());
		$customer->setGender(                 $this->getGender());
		$customer->setNewsletterSubscription( $this->getNewsletterSubscription());
		$customer->setCustomerGroups(         $this->getCustomerGroups());
		$customer->setAddresses(              $this->getAddresses());
		
		return $customer->utf8Encode($this->config->getEncoding());
	}
	
	/**
	 * @return string
	 */
	private function getCustomerId() {
		return $this->oxUser->oxuser__oxid->value;
	}
	
	/**
	 * @return string
	 */
	private function getCustomerToken() {
		return $this->oxUser->oxuser__oxid->value;
	}
	
	/**
	 * @return string
	 */
	private function getCustomerNumber() {
		return $this->oxUser->oxuser__oxcustnr->value;
	}
	
	/**
	 * @return string
	 */
	private function getFirstName() {
		return $this->oxUser->oxuser__oxfname->value;
	}
	
	/**
	 * @return string
	 */
	private function getLastName() {
		return $this->oxUser->oxuser__oxlname->value;
	}
	
	/**
	 * @return string
	 */
	private function getMail() {
		return $this->oxUser->oxuser__oxusername->value;
	}
	
	/**
	 * @return string
	 */
	private function getPhone() {
		return $this->oxUser->oxuser__oxfon->value;
	}
	
	/**
	 * @return string
	 */
	private function getMobile() {
		return $this->oxUser->oxuser__oxmobfon->value;
	}
	
	/**
	 * @return string
	 */
	private function getGender() {
		return marm_shopgate::getGenderByOxidSalutation($this->oxUser->oxuser__oxsal->value);
	}
	
	/**
	 * @return bool
	 */
	private function getNewsletterSubscription() {
		return $this->oxUser->getNewsSubscription()->oxnewssubscribed__oxdboptin->value;
	}
	
	/**
	 * @return ShopgateCustomerGroup[]
	 */
	private function getCustomerGroups() {
		$customerGroups = array();
		foreach (marm_shopgate::getUserGroupsByUser($this->oxUser) as $oxGroup) {
			$group = new ShopgateCustomerGroup();
			$group->setId($oxGroup->oxgroups__oxid->value);
			$group->setName($oxGroup->oxgroups__oxtitle->value);
			$customerGroups[$oxGroup->oxgroups__oxid->value] = $group;
		}
		return $customerGroups;
	}
	
	/**
	 * @return ShopgateAddress[]
	 */
	private function getAddresses() {
		$addresses = array();
		
		// the address data from oxuser are the invoice address
		// this address can also act as a delivery address
		$addresses[] = $this->buildShopgateAddress($this->oxUser);
		foreach ($this->oxUser->getUserAddresses() as $oAddress) {
			$addresses[] = $this->buildShopgateAddress($oAddress);
		}
		return $addresses;
	}
	
	/**
	 * create ShopgateAddress from oxaddress or oxuser
	 *
	 * @param oxUser|oxAddress $oxAddress
	 *
	 * @return ShopgateAddress
	 */
	private function buildShopgateAddress($oxAddress) {
		$tableName = $oxAddress->getCoreTableName();
		$country   = marm_shopgate::getCountryCodeByOxid($oxAddress->{"{$tableName}__oxcountryid"}->value);
		$state     = marm_shopgate::getStateCodeByOxid($oxAddress->{"{$tableName}__oxstateid"}->value);
		
		$result = new ShopgateAddress();
		
		$result->setId($oxAddress->{"{$tableName}__oxid"}->value);
		$result->setIsInvoiceAddress(($tableName == "oxuser"));
		$result->setIsDeliveryAddress(true);
		
		$result->setFirstName(	$oxAddress->{"{$tableName}__oxfname"}->value);
		$result->setLastName(	$oxAddress->{"{$tableName}__oxlname"}->value);
		$result->setCompany(	$oxAddress->{"{$tableName}__oxcompany"}->value);
		$result->setStreet1(	$oxAddress->{"{$tableName}__oxstreet"}->value . ' ' . $oxAddress->{"{$tableName}__oxstreetnr"}->value);
		$result->setStreet2(	$oxAddress->{"{$tableName}__oxaddinfo"}->value);
		$result->setCity(		$oxAddress->{"{$tableName}__oxcity"}->value);
		$result->setZipcode(	$oxAddress->{"{$tableName}__oxzip"}->value);
		$result->setCountry(	$country);
		$result->setState(		(!empty($country) && !empty($state)) ? "$country-$state" : '');
		$result->setGender(		marm_shopgate::getGenderByOxidSalutation($oxAddress->{"{$tableName}__oxsal"}->value));
		$result->setBirthday(	$oxAddress->{"{$tableName}__oxbirthdate"}->value);
		$result->setPhone(		$oxAddress->{"{$tableName}__oxfon"}->value);
		$result->setMobile(		$oxAddress->{"{$tableName}__oxmobfon"}->value);
		
		return $result;
	}
	
}