<?php
/**
 * This Plugin is subject to copyright protection. It is only for the use of
 * Shopgate GmbH customers, for the purpose of facilitating communication between
 * the IT systems of the customer and the IT systems of Shopgate GmbH via
 * www.Shopgate.com. Any reproduction, dissemination, public propagation,
 * processing or transfer to third parties is only permitted where we previously
 * consented thereto in writing. The provisions of paragraph 69 d,
 * sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of
 * the German Copyright Act shall remain unaffected.
 *
 *
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von
 * Kunden der Shopgate GmbH zum Zwecke der eigenen Kommunikation zwischen
 * dem IT-System des Kunden mit dem IT-System der Shopgate GmbH verwendet werden.
 * Eine darüber hinausgehende Vervielfältigung, Verbreitung, öffentliche
 * Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer
 * vorherigen schriftlichen Zustimmung zulässig. Die Regelungen
 * der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 */

/**
 * Class ShopgateShippingHelper
 */
class ShopgateShippingHelper extends ShopgateObject {

	const SHIPPING_SERVICE_ID_MOBILE_SHIPPING = 'mobile_shipping';

	/**
	 * @param ShopgateCartBase $cartOrOrder
	 *
	 * @return string
	 */
	public function getShippingServiceId(ShopgateCartBase $cartOrOrder) {
		if ($cartOrOrder->getShippingType() == 'PLUGINAPI') {
			return $cartOrOrder->getShippingInfos()->getName();
		}

		$group = $cartOrOrder->getShippingGroup();

		/** @var oxCountry $oxCountry */
		$oxCountry = oxNew('oxCountry');

		/** @var oxDeliverySet $oxDeliverySet */
		$oxDeliverySet = oxNew('oxDeliverySet');

		// get shipping set from database and optional iso country code
		$sql = "
				SELECT `ds`.`OXID`, `c`.`OXISOALPHA2`
				FROM `{$oxDeliverySet->getViewName()}` ds
				LEFT JOIN `oxobject2delivery` o2d ON ( o2d.oxdeliveryid = ds.oxid )
				LEFT JOIN `{$oxCountry->getViewName()}` c ON ( o2d.oxobjectid = c.oxid )
				WHERE `ds`.`shopgate_service_id` = ?
				  AND ( `c`.`OXISOALPHA2` = ? OR o2d.oxid IS NULL )";

		$country = null;
		if ($cartOrOrder->getDeliveryAddress() && $cartOrOrder->getDeliveryAddress()->getCountry()) {
			$country = $cartOrOrder->getDeliveryAddress()->getCountry();
		} else if ($cartOrOrder->getInvoiceAddress() && $cartOrOrder->getInvoiceAddress()->getCountry()) {
			$country = $cartOrOrder->getInvoiceAddress()->getCountry();
		}
		$aSetList = marm_shopgate::dbGetAll($sql, array($group, $country));

		if (!empty($aSetList)) {
			foreach ($aSetList as $aSetRow) {
				if ($aSetRow['OXISOALPHA2']) {
					// if iso code is available => use this and break
					return $aSetRow['OXID'];
				}
			}
		}

		return self::SHIPPING_SERVICE_ID_MOBILE_SHIPPING;
	}

	/**
	 * Activates our already existing DeliverySet (which is created by installing our module and by default deactivated) for a few seconds
	 * so it will be recognized by Oxid when we want to calculateBasket()
	 *
	 * @param int $seconds
	 */
	public function activateDeliverySet($seconds) {
		/** @var OxDeliverySet $oxShipping */
		$oxDeliverySet = oxNew('oxDeliverySet');
		if ($oxDeliverySet->load(ShopgateShippingHelper::SHIPPING_SERVICE_ID_MOBILE_SHIPPING)) {
			$oxDeliverySet->oxdeliveryset__oxactivefrom = new oxField(date("Y-m-d H:i:s", marm_shopgate::getOxUtilsDate()->getTime() - 1), oxField::T_RAW);
			$oxDeliverySet->oxdeliveryset__oxactiveto   =
				new oxField(date("Y-m-d H:i:s", marm_shopgate::getOxUtilsDate()->getTime() + $seconds), oxField::T_RAW);
			$oxDeliverySet->save();
		}
	}

	/**
	 *Deactivates our already existing DeliverySet
	 */
	public function deactivateDeliverySet() {
		/** @var OxDeliverySet $oxShipping */
		$oxDeliverySet = oxNew('oxDeliverySet');
		if ($oxDeliverySet->load(ShopgateShippingHelper::SHIPPING_SERVICE_ID_MOBILE_SHIPPING)) {
			$oxDeliverySet->oxdeliveryset__oxactivefrom = new oxField("0000-00-00 00:00:00", oxField::T_RAW);
			$oxDeliverySet->oxdeliveryset__oxactiveto   = new oxField("0000-00-00 00:00:00", oxField::T_RAW);
			$oxDeliverySet->save();
		}
	}

	/**
	 * @param ShopgateCartBase $shopgateOrder
	 *
	 * @return oxDelivery
	 */
	public function updateDeliveryEntry(ShopgateCartBase $shopgateOrder) {
		/** @var OxDelivery $oxDelivery */
		$oxDelivery = oxNew('oxDelivery');

		if (!$oxDelivery->load(ShopgateShippingHelper::SHIPPING_SERVICE_ID_MOBILE_SHIPPING)) {
			$oxDelivery = $this->createDeliveryEntry($shopgateOrder);
			$this->createMappingDeliverySetToDelivery($oxDelivery);
		} else {
			$this->updateDeliveryFields($shopgateOrder, $oxDelivery);
			$oxDelivery->save();
		}

		return $oxDelivery;
	}

	/**
	 * @param ShopgateCartBase $shopgateOrder
	 *
	 * @return oxDelivery
	 */
	public function createDeliveryEntry(ShopgateCartBase $shopgateOrder) {
		/** @var oxDelivery $oxDelivery */
		$oxDelivery                           = oxNew('oxDelivery');
		$oxDelivery->oxdelivery__oxid         = new oxField(ShopgateShippingHelper::SHIPPING_SERVICE_ID_MOBILE_SHIPPING, oxField::T_RAW);
		$oxDelivery->oxdelivery__oxactive     = new oxField(1, oxField::T_RAW);
		$oxDelivery->oxdelivery__oxtitle      = new oxField('Shopgate Mobile Shipping', oxField::T_RAW);
		$oxDelivery->oxdelivery__oxaddsumtype = new oxField('abs', oxField::T_RAW);
		$oxDelivery->oxdelivery__oxdeltype    = new oxField('p', oxField::T_RAW);
		$oxDelivery->oxdelivery__oxsort       = new oxField(99999, oxField::T_RAW);
		$oxDelivery->oxdelivery__oxfinalize   = new oxField(1, oxField::T_RAW);

		$this->updateDeliveryFields($shopgateOrder, $oxDelivery);

		$oxDelivery->save();

		marm_shopgate::dbExecute(
			"UPDATE oxdelivery SET oxid = '" . self::SHIPPING_SERVICE_ID_MOBILE_SHIPPING . "' WHERE oxid = '$oxDelivery->oxdelivery__oxid'"
		);

		$oxDelivery->oxdelivery__oxid = self::SHIPPING_SERVICE_ID_MOBILE_SHIPPING;

		return $oxDelivery;
	}

	/**
	 * Manipulates the parameter $oxDelivery by setting the oxparam, oxparamend and oxaddsum
	 *
	 * @param ShopgateCartBase $shopgateOrder
	 * @param OxDelivery       $oxDelivery
	 */
	public function updateDeliveryFields(ShopgateCartBase $shopgateOrder, $oxDelivery) {
		$oxDelivery->oxdelivery__oxparam    = new oxField(
			floor($shopgateOrder->getAmountItems()),
			oxField::T_RAW
		);
		$oxDelivery->oxdelivery__oxparamend = new oxField(
			ceil($shopgateOrder->getAmountItems()),
			oxField::T_RAW
		);
		$oxDelivery->oxdelivery__oxaddsum   = new oxField($shopgateOrder->getAmountShipping(), oxField::T_RAW);
	}

	/**
	 * @param oxDelivery $oxDelivery
	 */
	public function createMappingDeliverySetToDelivery($oxDelivery) {
		$oDel2delset = oxNew('oxbase');
		$oDel2delset->init('oxdel2delset');
		$oDel2delset->oxdel2delset__oxdelid    = new oxField($oxDelivery->oxdelivery__oxid);
		$oDel2delset->oxdel2delset__oxdelsetid = new oxField(ShopgateShippingHelper::SHIPPING_SERVICE_ID_MOBILE_SHIPPING);
		$oDel2delset->save();
	}
}