<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace unit\helpers\export;


class ItemTest extends \PHPUnit_Framework_TestCase {

	/**
	 * See OXID-339 for more information
	 */
	public function testTierPricesRoundingError() {
		$oxArticleMock = $this->getOxArticleMock(array('loadAmountPriceInfo'));
		$sut = $this->getShopgateItemExportHelper(array('getUnitAmountNet', 'getVpe', 'getPriceForQuantityAndGroup',));

		$sut->expects($this->any())->method('getUnitAmountNet')->will($this->returnValue(3.3193));
		$sut->expects($this->any())->method('getPriceForQuantityAndGroup')->will($this->returnValue(3.319327731));

		$oxArticleMock->expects($this->any())->method('loadAmountPriceInfo')->will($this->returnValue(array()));

		$tierPrices = $sut->getTierPrices($oxArticleMock);

		$this->assertEquals(true, empty($tierPrices));
	}

	public function testCustomerGroupPrice() {
		$sut = $this->getShopgateItemExportHelper(array('getUnitAmountNet', 'getVpe', 'getPriceForQuantityAndGroup',));

		$sut->expects($this->any())->method('getUnitAmountNet')->will($this->returnValue(3.3193));
		$sut->expects($this->any())->method('getPriceForQuantityAndGroup')->will($this->returnValue(2.50));

		$oxArticleMock = $this->getOxArticleMock(array('loadAmountPriceInfo'));
		$oxArticleMock->expects($this->any())->method('loadAmountPriceInfo')->will($this->returnValue(array()));

		$tierPrices = $sut->getTierPrices($oxArticleMock);

		$this->assertEquals(true, is_array($tierPrices));

		foreach ($tierPrices as $tierPrice) {
			$this->assertEquals('0.8193', $tierPrice->getReduction());
			$this->assertEquals('fixed', $tierPrice->getReductionType());
		}
	}

	public function testWeightUnitNotEmpty() {
		$sut = $this->getShopgateItemExportHelper(array('__construct'));

		$oxArticle = $this->getOxArticleMock(array('__construct'));

		$this->assertNotEmpty($sut->getWeightUnit($oxArticle));
	}

	/**
	 * @param array $methods
	 *
	 * @return \oxArticle|\PHPUnit_Framework_MockObject_MockObject
	 */
	private function getOxArticleMock(array $methods = array()) {
		return $this->getMockBuilder('oxArticle')
			->setMethods($methods)
			->getMock();
	}

	/**
	 * @param array $methods
	 *
	 * @return \ShopgateItemExportHelper|\PHPUnit_Framework_MockObject_MockObject
	 */
	private function getShopgateItemExportHelper(array $methods = array()) {
		return $this->getMockBuilder('ShopgateItemExportHelper')
			->disableOriginalConstructor()
			->setMethods($methods)
			->getMock();
	}
}
