<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace unit\helpers\payment\payone;

class UtilityTest extends \PHPUnit_Framework_TestCase {

	/** @var \ShopgatePaymentHelperPayoneUtility */
	private $subjectUnderTest;

	public function setUp() {
		$this->subjectUnderTest = $this->getMockBuilder('ShopgatePaymentHelperPayoneUtility')
			->disableOriginalConstructor()
			->setMethods(null)
			->getMock();
	}

	/**
	 * @param string $expectedExpiryDate
	 * @param string $year
	 * @param string $month
	 *
	 * @dataProvider provideDatesForExpiryDateGeneration
	 */
	public function testGeneratePayoneCreditCardExpiryDate($expectedExpiryDate, $year, $month) {
		$payonePaymentInfos = $this->generateShopgatePayonePaymentInfos(
			array(
				'credit_card' => array(
					'expiry_year'  => $year,
					'expiry_month' => $month,
				),
			)
		);
		$generateExpiryDate = $this->subjectUnderTest->generatePayoneCreditCardExpiryDate($payonePaymentInfos);

		$this->assertEquals($expectedExpiryDate, $generateExpiryDate);
	}

	/**
	 * @return array
	 */
	public function provideDatesForExpiryDateGeneration() {
		return array(
			'standard case'       => array('1612', '2016', '12',),
			'month only one char' => array('1601', '2016', '1',),
			'empty month'         => array('', '2016', '',),
			'month is null'       => array('', '2016', null,),
			'empty year'          => array('', '', '1',),
			'year is null'        => array('', null, '1',),
		);
	}

	/**
	 * @param array $paymentInfos
	 *
	 * @return \ShopgatePayonePaymentInfos
	 */
	private function generateShopgatePayonePaymentInfos(array $paymentInfos) {
		return new \ShopgatePayonePaymentInfos($paymentInfos, 2);
	}
}
