<?php
/**
 * This Plugin is subject to copyright protection. It is only for the use of
 * Shopgate GmbH customers, for the purpose of facilitating communication between
 * the IT systems of the customer and the IT systems of Shopgate GmbH via
 * www.Shopgate.com. Any reproduction, dissemination, public propagation,
 * processing or transfer to third parties is only permitted where we previously
 * consented thereto in writing. The provisions of paragraph 69 d,
 * sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of
 * the German Copyright Act shall remain unaffected.
 *
 *
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von
 * Kunden der Shopgate GmbH zum Zwecke der eigenen Kommunikation zwischen
 * dem IT-System des Kunden mit dem IT-System der Shopgate GmbH verwendet werden.
 * Eine darüber hinausgehende Vervielfältigung, Verbreitung, öffentliche
 * Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer
 * vorherigen schriftlichen Zustimmung zulässig. Die Regelungen
 * der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 */

/**
 * Admin controller for Shopgate config tab
 */
class shopgate_order extends oxAdminDetails {
	
	/**
	 * shopgate configuration template
	 *
	 * @var string
	 */
	protected $_sThisTemplate = 'shopgate_order.tpl';
	
	/**
	 * stores array for shopgate config, with information how to display it
	 *
	 * @var array
	 */
	protected $_aShopgateConfig = null;
	
	protected $isError = false;
	protected $errorMessage = '';
	
	public function syncorder() {
		$orderId = marm_shopgate::getRequestParameter('oxid');
		/** @var oxOrderShopgate $order */
		$order = oxNew('oxordershopgate');
		
		if ($order->load($orderId, 'oxorderid')) {
			if (!$order->syncFromShopgate()) {
				$this->isError      = true;
				$this->errorMessage = 'Error on sync order with Shopgate';
			}
		}
	}
	
	public function link_order() {
		$shopgateOrderNumber = marm_shopgate::getRequestParameter('shopgate_order_number');
		if (!$shopgateOrderNumber) {
			$this->isError      = true;
			$this->errorMessage = "Missing Shopgate order number";
			
			return;
		}
		
		$orderId = marm_shopgate::getRequestParameter('oxid');
		
		/** @var oxOrderShopgate $order */
		$order = oxNew('oxordershopgate');
		
		if ($order->load($shopgateOrderNumber, "order_number")) {
			$this->isError      = true;
			$this->errorMessage = "Order already exists!";
			
			if (($oxOrder = $order->getOxidOrder())) {
				$this->errorMessage .= " Order number: {$oxOrder->oxorder__oxordernr->value}";
			}
			
			return;
		}
		
		$order->oxordershopgate__order_number = new oxField($shopgateOrderNumber, oxField::T_RAW);
		$order->oxordershopgate__oxorderid    = new oxField($orderId, oxField::T_RAW);
		$order->save();
		
		$this->syncorder();
	}
	
	public function unlink_order() {
		$orderId = marm_shopgate::getRequestParameter('oxid');
		
		/** @var oxOrderShopgate $order */
		$order = oxNew('oxordershopgate');
		$order->load($orderId, 'oxorderid');
		$order->delete();
	}
	
	public function reset() {
		$orderId = marm_shopgate::getRequestParameter('oxid');
		
		/** @var oxOrderShopgate $order */
		$order = oxNew('oxordershopgate');
		if ($order->load($orderId, 'oxorderid')) {
			$order->oxordershopgate__is_sent_to_shopgate = new oxField("0", oxField::T_RAW);
			$order->save();
		}
	}
	
	public function getShopgateOrder($blReset = false) {
		$id = $this->getEditObjectId();
		
		/** @var oxOrderShopgate $order */
		$order = oxNew('oxordershopgate');
		
		if ($order->load($id, 'oxorderid') && !$order->oxordershopgate__order_data->value) {
			$this->syncorder();
		}
		
		return $order;
	}
	
	public function getIsError() {
		return $this->isError;
	}
	
	public function getErrorMessage() {
		return $this->errorMessage;
	}
	
	public function getIsShopgateOrder() {
		$id = $this->getEditObjectId();
		/** @var oxOrderShopgate $order */
		$order = oxNew('oxordershopgate');
		
		return $order->load($id, 'oxorderid');
	}
}