<?php
/**
 * This Plugin is subject to copyright protection. It is only for the use of
 * Shopgate GmbH customers, for the purpose of facilitating communication between
 * the IT systems of the customer and the IT systems of Shopgate GmbH via
 * www.Shopgate.com. Any reproduction, dissemination, public propagation,
 * processing or transfer to third parties is only permitted where we previously
 * consented thereto in writing. The provisions of paragraph 69 d,
 * sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of
 * the German Copyright Act shall remain unaffected.
 *
 *
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von
 * Kunden der Shopgate GmbH zum Zwecke der eigenen Kommunikation zwischen
 * dem IT-System des Kunden mit dem IT-System der Shopgate GmbH verwendet werden.
 * Eine darüber hinausgehende Vervielfältigung, Verbreitung, öffentliche
 * Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer
 * vorherigen schriftlichen Zustimmung zulässig. Die Regelungen
 * der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 */

/**
 * Admin controller for Shopgate config tab
 */
class shopgate_shipping extends oxAdminDetails {
	
	/**
	 * shopgate configuration template
	 *
	 * @var string
	 */
	protected $_sThisTemplate = 'shopgate_shipping.tpl';
	
	/**
	 * stores array for shopgate config, with information how to display it
	 *
	 * @var array
	 */
	protected $_aShopgateConfig = null;
	
	protected $isError = false;
	protected $errorMessage = "";
	
	public function render() {
		$return = parent::render();
		
		$soxId = $this->_aViewData['oxid'] = $this->getEditObjectId();
		if ($soxId != "-1" && isset($soxId)) {
			// load object
			/** @var oxDeliverySet $odeliveryset */
			$odeliveryset = oxNew("oxdeliveryset");
			$odeliveryset->loadInLang($this->_iEditLang, $soxId);
			
			$oOtherLang = $odeliveryset->getAvailableInLangs();
			
			if (!isset($oOtherLang[$this->_iEditLang])) {
				// echo "language entry doesn't exist! using: ".key($oOtherLang);
				$odeliveryset->loadInLang(key($oOtherLang), $soxId);
			}
			
			$this->_aViewData['edit'] = $odeliveryset;
		}
		
		$this->_aViewData['delivery_services'] = $this->getDeliveryServiceList();
		
		return $return;
	}
	
	public function setDeliveryService() {
		/** @var oxDeliverySet $oDel */
		$oDel = oxNew('oxdeliveryset');
		
		// Works with oxid 4.3
		$fields = $oDel->getSelectFields();
		$fields = preg_replace("/`/", "", $fields);
		$fields = explode(", ", $fields);
		
		if (!in_array("{$oDel->getViewName()}.shopgate_service_id", $fields)) {
			$this->isError      = true;
			$this->errorMessage = "Cannot save - The field 'shopgate_service_id' is missing";
			
			return;
		}
		
		$sOxid                      = marm_shopgate::getRequestParameter('oxid');
		$sShopgateShippingServiceId = marm_shopgate::getRequestParameter('shopgate_shipping_service_id');
		
		$oDel->load($sOxid);
		
		$oDel->oxdeliveryset__shopgate_service_id = new oxField($sShopgateShippingServiceId, oxField::T_RAW);
		$oDel->save();
	}
	
	public function getDeliveryServiceList() {
		return array(
			'DHL', 'DHLEXPRESS', 'DP',
			'DPD', 'FEDEX', 'GLS',
			'HLG', 'TNT', 'TOF',
			'UPS', 'USPS', 'OTHER',
		);
	}
	
	
	public function getIsError() {
		return $this->isError;
	}
	
	public function getErrorMessage() {
		return $this->errorMessage;
	}
	
}