<?php
/**
 * This Plugin is subject to copyright protection. It is only for the use of
 * Shopgate GmbH customers, for the purpose of facilitating communication between
 * the IT systems of the customer and the IT systems of Shopgate GmbH via
 * www.Shopgate.com. Any reproduction, dissemination, public propagation,
 * processing or transfer to third parties is only permitted where we previously
 * consented thereto in writing. The provisions of paragraph 69 d,
 * sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of
 * the German Copyright Act shall remain unaffected.
 *
 *
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von
 * Kunden der Shopgate GmbH zum Zwecke der eigenen Kommunikation zwischen
 * dem IT-System des Kunden mit dem IT-System der Shopgate GmbH verwendet werden.
 * Eine darüber hinausgehende Vervielfältigung, Verbreitung, öffentliche
 * Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer
 * vorherigen schriftlichen Zustimmung zulässig. Die Regelungen
 * der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 */

class ShopgateUnknownOxidConfigFields {

	const OXID_VARTYPE_STRING = 'str';

	private static $oxidConfigUnknownField = 'unknown_fields';

	/** @var array */
	private $unknownOxidConfigFields;

	/** @var \oxConfig */
	private $oxidConfig;

	/** @var ShopgateConfigOxid */
	private $shopgateConfig;

	/** @var marm_shopgate */
	private $marmShopgate;

	/**
	 * @param ShopgateConfigOxid $shopgateConfigOxid
	 * @param \oxConfig          $oxConfig
	 * @param marm_shopgate      $marmShopgate
	 */
	public function __construct(ShopgateConfigOxid $shopgateConfigOxid, $oxConfig, $marmShopgate) {
		$this->unknownOxidConfigFields = array();
		$this->shopgateConfig          = $shopgateConfigOxid;
		$this->oxidConfig              = $oxConfig;
		$this->marmShopgate            = $marmShopgate;
	}

	/**
	 * @param array $unknownOxidConfigFields
	 */
	public function save(array $unknownOxidConfigFields) {
		$unknownOxidConfigFields = $this->setAlreadyExistingUnknownFields($unknownOxidConfigFields);

		$this->oxidConfig->saveShopConfVar(
			self::OXID_VARTYPE_STRING,
			$this->marmShopgate->getOxidConfigKey(self::$oxidConfigUnknownField),
			$this->shopgateConfig->jsonEncode($unknownOxidConfigFields)
		);
	}

	/**
	 * @post $this->shopgateConfig contains all "unknown fields" as object variables
	 */
	public function load() {
		$unknownOxidConfigFieldsJsonString = $this->oxidConfig->getConfigParam(
			$this->marmShopgate->getOxidConfigKey(self::$oxidConfigUnknownField)
		);

		if (empty($unknownOxidConfigFieldsJsonString)) {
			return;
		}

		$unknownOxidConfigurationFields = $this->shopgateConfig->jsonDecode($unknownOxidConfigFieldsJsonString, true);
		foreach ($unknownOxidConfigurationFields as $field => $value) {
			$setter = $this->shopgateConfig->camelize($field);
			$methodName = 'set' . $setter;
			if (method_exists($this->shopgateConfig, $methodName)) {
				$this->shopgateConfig->$methodName($value);
			}
		}

		$this->unknownOxidConfigFields = array_keys($unknownOxidConfigurationFields);
	}

	/**
	 * @param array $unknownOxidConfigFields
	 *
	 * @return array
	 */
	private function setAlreadyExistingUnknownFields(array $unknownOxidConfigFields) {
		foreach ($this->unknownOxidConfigFields as $alreadyExistingOxidConfigUnknownField) {
			if (isset($unknownOxidConfigFields[$alreadyExistingOxidConfigUnknownField])) {
				continue;
			}

			$getter                                                          = $this->shopgateConfig->camelize(
				$alreadyExistingOxidConfigUnknownField
			);
			$unknownOxidConfigFields[$alreadyExistingOxidConfigUnknownField] = $this->shopgateConfig->{'get' . $getter}();
		}

		return $unknownOxidConfigFields;
	}
}