<?php
/**
 * This Plugin is subject to copyright protection. It is only for the use of
 * Shopgate GmbH customers, for the purpose of facilitating communication between
 * the IT systems of the customer and the IT systems of Shopgate GmbH via
 * www.Shopgate.com. Any reproduction, dissemination, public propagation,
 * processing or transfer to third parties is only permitted where we previously
 * consented thereto in writing. The provisions of paragraph 69 d,
 * sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of
 * the German Copyright Act shall remain unaffected.
 *
 *
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von
 * Kunden der Shopgate GmbH zum Zwecke der eigenen Kommunikation zwischen
 * dem IT-System des Kunden mit dem IT-System der Shopgate GmbH verwendet werden.
 * Eine darüber hinausgehende Vervielfältigung, Verbreitung, öffentliche
 * Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer
 * vorherigen schriftlichen Zustimmung zulässig. Die Regelungen
 * der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 */

/**
 * Class ShopgateVoucherHelper
 */
class ShopgateVoucherHelper extends ShopgateObject {
	
	const VOUCHER_OXID = 'shopgate_voucher';
	
	/**
	 * @param oxVoucher $oxVoucher
	 *
	 * @return bool
	 */
	public function isVoucherAlreadyUsed(oxVoucher $oxVoucher) {
		if (_SHOPGATE_ACTION == ShopgatePluginOxid::ACTION_ADD_ORDER) {
			// In add_order we have no choice but to use the given coupon, even if it has already been used.
			// Otherwise we might not be able to import the order.
			return false;
		}
		$dateUsed = (int) str_replace('-', '', $oxVoucher->oxvouchers__oxdateused->value);
		
		return !empty($dateUsed);
	}
	
	/**
	 * @return oxVoucherSerie
	 *
	 * @throws ShopgateLibraryException
	 */
	public function createVoucherSeriesForShopgate() {
		$this->log("Create voucher series", ShopgateLogger::LOGTYPE_DEBUG);
		$voucherId = self::VOUCHER_OXID;
		
		/** @var oxVoucherSerie $oxVoucherSeries */
		$oxVoucherSeries = oxNew('oxVoucherSerie');
		$shopId          = marm_shopgate::getOxConfig()->getShopId();
		
		$qry = "INSERT INTO `{$oxVoucherSeries->getViewName()}`
				(`oxid`, `oxshopid`, `oxserienr`, `oxseriedescription`, `oxdiscounttype`)
		 VALUES ('{$voucherId}', '{$shopId}', 'Shopgate Gutscheine', 'Shopgate Gutscheine', 'absolute')";
		marm_shopgate::dbExecute($qry);
		
		$qry    = "SELECT * FROM `{$oxVoucherSeries->getViewName()}` o WHERE o.oxid = '{$voucherId}'";
		$result = marm_shopgate::dbGetAll($qry);
		
		if (isset($result[0]['OXMAPID'])) {
//            there is an additional table "oxvoucherseries2shop" in newer Oxid versions
//            without an entry there, the series won't be found
			$table = $oxVoucherSeries->getViewName() . '2shop';
			$qry   = "INSERT INTO `{$table}`
				(`oxshopid`, `oxmapobjectid`)
		 	 VALUES ('{$shopId}', '{$result[0]['OXMAPID']}')";
			
			marm_shopgate::dbExecute($qry);
		}
		
		if (!$oxVoucherSeries->load($voucherId)) {
			throw new ShopgateLibraryException(ShopgateLibraryException::UNKNOWN_ERROR_CODE, "Cannot create voucher serie", true);
		}
		
		$oxVoucherSeries->oxvoucherseries__oxallowsameseries  = new oxField(1, oxField::T_RAW);
		$oxVoucherSeries->oxvoucherseries__oxallowotherseries = new oxField(1, oxField::T_RAW);
		$oxVoucherSeries->oxvoucherseries__oxallowuseanother  = new oxField(1, oxField::T_RAW);
		$oxVoucherSeries->oxvoucherseries__oxminimumvalue     = new oxField(0, oxField::T_RAW);
		$oxVoucherSeries->oxvoucherseries__oxcalculateonce    = new oxField(0, oxField::T_RAW);
		$oxVoucherSeries->save();
		
		return $oxVoucherSeries;
	}
}