<?php
/**
 * This Plugin is subject to copyright protection. It is only for the use of
 * Shopgate GmbH customers, for the purpose of facilitating communication between
 * the IT systems of the customer and the IT systems of Shopgate GmbH via
 * www.Shopgate.com. Any reproduction, dissemination, public propagation,
 * processing or transfer to third parties is only permitted where we previously
 * consented thereto in writing. The provisions of paragraph 69 d,
 * sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of
 * the German Copyright Act shall remain unaffected.
 *
 *
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von
 * Kunden der Shopgate GmbH zum Zwecke der eigenen Kommunikation zwischen
 * dem IT-System des Kunden mit dem IT-System der Shopgate GmbH verwendet werden.
 * Eine darüber hinausgehende Vervielfältigung, Verbreitung, öffentliche
 * Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer
 * vorherigen schriftlichen Zustimmung zulässig. Die Regelungen
 * der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 */

class marm_shopgate_oxorder extends marm_shopgate_oxorder_parent {
	
	/**
	 * Usually defined in parent class, but missing in Oxid 4.1
	 */
	const ORDER_STATE_MAILINGERROR = 0;
	
	/**
	 * Usually defined in parent class, but missing in Oxid 4.1 
	 */
	const ORDER_STATE_OK = 1;
	
	/**
	 * Delete the entry for shopgate order data if oxorder will delete
	 *
	 * @param string $sOXID
	 * @return boolean
	 * @see oxOrder::delete
	 */
	public function delete($sOXID = null) {
		parent::delete($sOXID);
		
		# you can't rely on the parent function returning true or false
		# cause some other modules override the function and don't return anything
		
		$oxOrderId = $sOXID ? $sOXID : $this->getId();
		if (!$this->load($oxOrderId)) {
			/** @var oxOrderShopgate $shopgateOrder */
			$shopgateOrder = oxNew("oxOrderShopgate");
			if ($shopgateOrder->load($oxOrderId, "oxorderid")) {
				$shopgateOrder->delete();
			}
			
			return true;
		}
		
		return false;
	}
	
	/**
	 * executes parent save, calls marmCheckSendDate, returns parent result
	 *
	 * @return boolean
	 * @see oxOrder::save
	 */
	public function save() {
		$blResult = parent::save();
		$this->_marmCheckSendDate();
		
		return $blResult;
	}
	
	/**
	 * if order send date is set, and order is from shopgate,
	 *
	 * @return void
	 */
	protected function _marmCheckSendDate() {
		$iTime = strtotime($this->oxorder__oxsenddate->value);
		
		/** @var oxOrderShopgate $shopgateOrder */
		$shopgateOrder = oxNew("oxOrderShopgate");
		
		if ($iTime > 0 && $shopgateOrder->load($this->getId(), "oxorderid")) {
			try {
				$shopgateOrder->confirmShipping();
			} catch (Exception $e) {
			}
		}
	}
	
	/**
	 * ignore basket and order validation
	 *
	 * @param oxbasket $oBasket
	 * @param oxuser   $oUser
	 * @return NULL|int
	 * @see oxOrder::validateOrder
	 */
	public function validateOrder($oBasket, $oUser) {
		if (defined("_SHOPGATE_API") && _SHOPGATE_API) {
			return null;
		}
		
		return parent::validateOrder($oBasket, $oUser);
	}
	
	/**
	 * @param oxUser        $oUser
	 * @param oxBasket      $oBasket
	 * @param oxUserPayment $oPayment
	 * @return number
	 * @see oxOrder::_sendOrderByEmail
	 */
	protected function _sendOrderByEmail($oUser = null, $oBasket = null, $oPayment = null) {
		$sendToUser  = marm_shopgate::getInstance()->getConfig()->getSendMails();
		$sendToOwner = marm_shopgate::getInstance()->getConfig()->getSendMailsToOwner();
		if (!defined("_SHOPGATE_API") || !_SHOPGATE_API || ($sendToUser && $sendToOwner)) {
			return parent::_sendOrderByEmail($oUser, $oBasket, $oPayment);
		}
		
		$this->_oUser = $oUser;
		$this->_oBasket = $oBasket;
		$this->_oPayment = $oPayment;
		
		/** @var oxEmail $oxEmail */
		$oxEmail = oxNew('oxEmail');
		
		if ($sendToOwner) {
			$oxEmail->sendOrderEMailToOwner($this);
		}
		
		if ($sendToUser && !$oxEmail->sendOrderEMailToUser($this)) {
			return self::ORDER_STATE_MAILINGERROR;
		}
		return self::ORDER_STATE_OK;
	}

	protected function _executePayment(oxBasket $oBasket, $oUserpayment) {
		if (defined("_SHOPGATE_API") && _SHOPGATE_API && defined("_SHOPGATE_ACTION") && _SHOPGATE_ACTION == 'add_order') {
			return true;
		}
		
		return parent::_executePayment($oBasket, $oUserpayment);
	}
}