<?php
/**
 * This Plugin is subject to copyright protection. It is only for the use of
 * Shopgate GmbH customers, for the purpose of facilitating communication between
 * the IT systems of the customer and the IT systems of Shopgate GmbH via
 * www.Shopgate.com. Any reproduction, dissemination, public propagation,
 * processing or transfer to third parties is only permitted where we previously
 * consented thereto in writing. The provisions of paragraph 69 d,
 * sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of
 * the German Copyright Act shall remain unaffected.
 *
 *
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von
 * Kunden der Shopgate GmbH zum Zwecke der eigenen Kommunikation zwischen
 * dem IT-System des Kunden mit dem IT-System der Shopgate GmbH verwendet werden.
 * Eine darüber hinausgehende Vervielfältigung, Verbreitung, öffentliche
 * Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer
 * vorherigen schriftlichen Zustimmung zulässig. Die Regelungen
 * der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 */

class Shopgate_Model_Export_Category extends Shopgate_Model_Catalog_Category {
	
	const ROOT_CATEGORY_ID = 'oxrootid';
	
	/** @var oxCategory */
	protected $item;
	
	/** @var ShopgateConfigOxid */
	private $config;
	
	/** @var int */
	private $maxSort;
	
	###############################################################################################
	## Setters for local variables
	###############################################################################################
	
	/**
	 * @param ShopgateConfigOxid $config
	 */
	public function setConfig(ShopgateConfigOxid $config) {
		$this->config = $config;
	}
	
	/**
	 * @param int $maxSort
	 */
	public function setMaxSort($maxSort) {
		$this->maxSort = $maxSort;
	}
	
	###############################################################################################
	## Setters for category data
	###############################################################################################
	
	public function setDeeplink() {
		$url = $this->parseUrl(marm_shopgate::cleanUrl($this->item->getLink()));
		parent::setDeeplink($url);
	}
	
	public function setImage() {
		$imageUrl = "";
		if (!empty($this->item->oxcategories__oxicon->value)) {
			$imageUrl = $this->item->getPictureUrl() . $this->item->oxcategories__oxicon->value;
		} else if (!empty($this->item->oxcategories__oxthumb->value)) {
			$imageUrl = $this->item->getPictureUrl() . $this->item->oxcategories__oxthumb->value;
		}
		
		$image = new Shopgate_Model_Media_Image();
		$image->setUrl($imageUrl);
		parent::setImage($image);
	}
	
	public function setIsActive() {
		$isActive = $this->item->oxcategories__oxactive->value == 1 && $this->item->oxcategories__oxhidden->value == 0;
		parent::setIsActive($isActive);
	}
	
	public function setIsAnchor() {
		parent::setIsAnchor(false);
	}
	
	public function setName() {
		parent::setName($this->item->oxcategories__oxtitle->value);
	}
	
	public function setParentUid() {
		$parentId = $this->item->oxcategories__oxparentid->value;
		if ($parentId == self::ROOT_CATEGORY_ID) {
			$parentId = "";
		}
		parent::setParentUid($parentId);
	}
	
	public function setSortOrder() {
		parent::setSortOrder($this->maxSort - $this->item->oxcategories__oxsort->value);
	}
	
	public function setUid() {
		parent::setUid($this->item->oxcategories__oxid->value);
	}
	
	###############################################################################################
	## Helpers
	###############################################################################################
	
	/**
	 * @param string $url
	 * @return string
	 */
	private function parseUrl($url) {
		if ($this->config->getHtaccessUser() && $this->config->getHtaccessPassword()) {
			$replacement = "http://";
			$replacement .= urlencode($this->config->getHtaccessUser());
			$replacement .= ":";
			$replacement .= urlencode($this->config->getHtaccessPassword());
			$replacement .= "@";
			
			$url = preg_replace("/^http:\/\//i", $replacement, $url, 1);
		}
		
		return $url;
	}
	
	###############################################################################################
	## Csv
	###############################################################################################
	
	/**
	 * @return array
	 */
	public function asCsv() {
		return array(
			'category_number' => $this->getUid(),
			'category_name'   => $this->getName(),
			'parent_id'       => $this->getParentUid(),
			'url_image'       => $this->getImage()->getUrl(),
			'order_index'     => $this->getSortOrder(),
			'is_active'       => $this->getIsActive() ? 1 : 0,
			'url_deeplink'    => $this->getDeeplink(),
		);
	}
}