<?php

/**
 * Shopgate GmbH
 *
 * URHEBERRECHTSHINWEIS
 *
 * Dieses Plugin ist urheberrechtlich geschützt. Es darf ausschließlich von Kunden der Shopgate GmbH
 * zum Zwecke der eigenen Kommunikation zwischen dem IT-System des Kunden mit dem IT-System der
 * Shopgate GmbH über www.shopgate.com verwendet werden. Eine darüber hinausgehende Vervielfältigung, Verbreitung,
 * öffentliche Zugänglichmachung, Bearbeitung oder Weitergabe an Dritte ist nur mit unserer vorherigen
 * schriftlichen Zustimmung zulässig. Die Regelungen der §§ 69 d Abs. 2, 3 und 69 e UrhG bleiben hiervon unberührt.
 *
 * COPYRIGHT NOTICE
 *
 * This plugin is the subject of copyright protection. It is only for the use of Shopgate GmbH customers,
 * for the purpose of facilitating communication between the IT system of the customer and the IT system
 * of Shopgate GmbH via www.shopgate.com. Any reproduction, dissemination, public propagation, processing or
 * transfer to third parties is only permitted where we previously consented thereto in writing. The provisions
 * of paragraph 69 d, sub-paragraphs 2, 3 and paragraph 69, sub-paragraph e of the German Copyright Act shall remain unaffected.
 *
 * @author Shopgate GmbH <interfaces@shopgate.com>
 */

namespace unit\helpers\export;

class UnknownOxidConfigFieldsTest extends \PHPUnit_Framework_TestCase {

	/** @var \ShopgateUnknownOxidConfigFields */
	private $subjectUnderTest;

	/** @var \oxConfig|\PHPUnit_Framework_MockObject_MockObject */
	private $oxidConfigMock;

	/** @var \ShopgateConfigOxid|\PHPUnit_Framework_MockObject_MockObject */
	private $shopgateConfigMock;

	/** @var \marm_shopgate|\PHPUnit_Framework_MockObject_MockObject */
	private $marmShopgateMock;

	public function setUp() {
		$this->oxidConfigMock = $this->getMockBuilder('\oxConfig')
			->setMethods(array('saveShopConfVar', 'getConfigParam'))
			->getMock();

		$this->shopgateConfigMock = $this->getMockBuilder('\ShopgateConfigOxid')
			->disableOriginalConstructor()
			->getMock();

		$this->marmShopgateMock = $this->getMockBuilder('marm_shopgate')
			->setMethods(array('getOxidConfigKey'))
			->disableOriginalConstructor()
			->getMock();

		$this->subjectUnderTest =
			new \ShopgateUnknownOxidConfigFields($this->shopgateConfigMock, $this->oxidConfigMock, $this->marmShopgateMock);
	}

	public function testRunSaveMethod() {
		$this->subjectUnderTest->save(array('html_tags' => 'test'));
	}

	public function testLoadConfig() {
		$this->oxidConfigMock->method('getConfigParam')->willReturn('{"html_tags" : "test"}');

		$this->shopgateConfigMock->expects($this->once())->method('jsonDecode')->with('{"html_tags" : "test"}', true)->willReturn(
			array('html_tags' => 'test')
		);
		$this->shopgateConfigMock->expects($this->once())->method('camelize')->with('html_tags')->willReturn('HtmlTags');

		$this->shopgateConfigMock->expects($this->once())->method('setHtmlTags')->with('test');

		$this->subjectUnderTest->load();
	}

	/**
	 * There is already a configuration value for 'html_tags' and 'encoding' which gets loaded.
	 * Two new configuration values should be saved 'html_tags' and 'shop_is_active'.
	 * We need to check the new value for html_tags, see that encoding is still there
	 */
	public function testRunSaveMethodAfterLoad() {
		$this->oxidConfigMock->method('getConfigParam')->willReturn('{"html_tags" : "test", "encoding" : "UTF-8"}');

		$this->shopgateConfigMock->expects($this->once())->method('jsonDecode')->with('{"html_tags" : "test", "encoding" : "UTF-8"}', true)
			->willReturn(array('html_tags' => 'test', 'encoding' => 'UTF-8'));
		$this->shopgateConfigMock->method('camelize')->willReturnOnConsecutiveCalls('HtmlTags', 'Encoding', 'Encoding');

		$this->shopgateConfigMock->expects($this->once())->method('setHtmlTags')->with('test');
		$this->shopgateConfigMock->expects($this->once())->method('setEncoding')->with('UTF-8');
		$this->shopgateConfigMock->expects($this->once())->method('getEncoding')->willReturn('UTF-8');

		$this->subjectUnderTest->load();


		$this->shopgateConfigMock->expects($this->once())->method('jsonEncode')->with(
			[
				'html_tags'      => '{"html_tags" : "test"}',
				'encoding'       => 'UTF-8',
				'shop_is_active' => 1,
			]
		);

		$this->subjectUnderTest->save(
			array(
				'shop_is_active' => 1,
				'html_tags'      => '{"html_tags" : "test"}',
			)
		);
	}


	public function testLoadConfigNotExistingFields() {
		$this->oxidConfigMock->method('getConfigParam')->willReturn('{"abc123" : "test"}');

		$this->shopgateConfigMock->expects($this->once())->method('jsonDecode')->with('{"abc123" : "test"}', true)->willReturn(
			array('abc123' => 'test')
		);
		$this->shopgateConfigMock->expects($this->once())->method('camelize')->with('abc123')->willReturn('Abc123');

		$this->subjectUnderTest->load();
	}
}
